
/*****************************************************************************
** Internal: AboutCommand()
*/

static ERROR AboutCommand(struct CommandLine *Self, STRING Args)
{
   return(CLI_About(Self, NULL));
}

/*****************************************************************************
** Internal: CreateCommand()
*/

static ERROR CreateCommand(struct CommandLine *Self, STRING Args)
{
   LONG i, j, ClassID, success;
   OBJECTID ObjectID;
   OBJECTPTR Object;
   BYTE ClassName[60];
   STRING buffer;

   if (!Args) return(ERR_Args);

   buffer   = Self->Buffer;
   ObjectID = NULL;
   i = 0;
   if (Args[i]) {
      j = 0;
      while ((Args[i]) AND (Args[i] != ' ')) buffer[j++] = Args[i++];
      buffer[j] = 0;

      StrCopy(buffer, ClassName, sizeof(ClassName));
      if ((ClassID = ResolveClassName(ClassName))) {
         NewObject(ClassID, NULL, NULL, &ObjectID);
      }
      else OutputText(Self, "The class name could not be resolved.");
   }
   else OutputText(Self, "You have used this command incorrectly.");

   if (!ObjectID) return(ERR_Failed);

   success = FALSE;
   if (AccessObject(ObjectID, 1000, &Object) IS ERR_Okay) {
      ProcessFields(Self, Args+i, Object);
      if (Action(AC_Init, Object, NULL) IS ERR_Okay) {
         Action(AC_OpeningTag, Object, NULL);
         Action(AC_ClosingTag, Object, NULL);
         success = TRUE;
      }
      else {
         OutputText(Self, "The object could not be initialised.\n");
         ActionMsg(AC_Free, ObjectID, NULL);
      }

      ReleaseObject(Object);

      /* After the ReleaseObject(), check if the object still exists, and
      ** if so, add it to our internal table of created objects.
      */

      if (CheckObjectExists(ObjectID, NULL) IS ERR_True) {
         StrFormat(Self->Buffer, sizeof(Self->Buffer), "The %s object has been created and is accessible through ID %d.", ClassName, ObjectID);
         OutputText(Self, Self->Buffer);

         CleanupObjectTable(Self);

         for (i=0; i < TABLE_SIZE; i++) {
            if (!Self->ObjectTable[i]) {
               Self->ObjectTable[i] = ObjectID;
               break;
            }
         }
      }
      else if (success IS TRUE) {
         OutputText(Self, "The create command was successfully processed and the object was destroyed.");
      }
   }
   else OutputText(Self, "A process has stolen the object that I allocated.\n");

   return(ERR_Okay);
}

/*****************************************************************************
** Internal: EraseCommand()
*/

static ERROR EraseCommand(struct CommandLine *Self, STRING Args)
{
   return(CLI_Erase(Self, NULL));
}

/*****************************************************************************
** Internal: ExpungeCommand()
*/

static ERROR ExpungeCommand(struct CommandLine *Self, STRING Args)
{
   OBJECTID SystemTaskID;

   if (FastFindObject("SystemTask", ID_TASK, &SystemTaskID, 1, NULL) IS ERR_Okay) {
      ActionMsg(MT_ExpungeTask, SystemTaskID, NULL);
      OutputText(Self, "The system has been expunged of redundant code.");
   }
   else OutputText(Self, "Failed to perform the expunge as the SystemTask is missing.");

   return(ERR_Okay);
}

/*****************************************************************************
** Internal: FreeCommand()
*/

static ERROR FreeCommand(struct CommandLine *Self, STRING Args)
{
   OBJECTID ObjectID, list[2];
   LONG ObjectCount;

   if (!Args) return(ERR_Args);

   if ((ObjectID = (OBJECTID)StrToInt(Args))) {
      ActionMsg(AC_Free, ObjectID, NULL);
   }
   else if (FastFindObject(Args, NULL, list, ARRAYSIZE(list), &ObjectCount) IS ERR_Okay) {
      if (ObjectCount > 1) {
         OutputText(Self, "There are multiple objects with that name.  You need to specify an ID.");
      }
      else {
         ObjectID = list[0];
         ActionMsg(AC_Free, ObjectID, NULL);
         OutputText(Self, "The object has been freed.");
      }
   }

   return(ERR_Okay);
}

/*****************************************************************************
** Internal: HelpCommand()
*/

static ERROR HelpCommand(struct CommandLine *Self, STRING Args)
{
   struct mtCommandHelp help;

   help.Command = Args;
   return(CLI_Help(Self, &help));
}

/*****************************************************************************
** Internal: InstanceCommand()
*/

static ERROR InstanceCommand(struct CommandLine *Self, STRING Args)
{
   LONG instance;
   UBYTE buffer[120];

   GetField(CurrentTask(), FID_Instance, FT_LONG, &instance);

   StrFormat(buffer, 120, "The CLI is working within instance #%d.", instance);
   OutputText(Self, buffer);
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: ScriptCommand()
*/

static ERROR ScriptCommand(struct CommandLine *Self, STRING Args)
{
   OBJECTPTR run;
   LONG j;

   if (!Args) return(ERR_Args);

   if (NewPrivateObject(ID_RUN, NULL, &run) IS ERR_Okay) {

      SetField(run, FID_Location, FT_POINTER, "bin:runscript");

      /*** Build the runscript arguments ***/

      j = StrCopy("-target desktop -wait -script ", Self->Buffer, COPY_ALL);

      /*** Extract the location of the script ***/

      if (*Args IS '"') {
         Args++;
         while ((*Args) AND (*Args != '"') AND (j < SIZE_BUFFER-1)) Self->Buffer[j++] = *Args++;
         if (*Args) Args++;
      }
      else while ((*Args) AND (*Args != ' ') AND (j < SIZE_BUFFER-1)) Self->Buffer[j++] = *Args++;

      /*** Copy the rest of the arguments into the buffer ***/

      while (*Args IS ' ') Args++;

      if (*Args) {
         j += StrCopy(" -args ", Self->Buffer + j, SIZE_BUFFER-j-1);
         while (*Args) Self->Buffer[j++] = *Args++;
      }

      Self->Buffer[j] = 0;

      SetField(run, FID_Args, FT_POINTER, Self->Buffer);

      if (Action(AC_Init, run, NULL) IS ERR_Okay) {
         Action(AC_Activate, run, NULL);
      }

      Action(AC_Free, run, NULL);
   }
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: StatusCommand()
*/

static ERROR StatusCommand(struct CommandLine *Self, STRING Args)
{
   return(CLI_Status(Self, NULL));
}
